/*  smplayer, GUI front-end for mplayer.
    Copyright (C) 2007 Ricardo Villalba <rvm@escomposlinux.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <qapplication.h>
#include <qcolor.h>
#include "gui.h"
#include "constants.h"
#include "helper.h"
#include "global.h"
#include "version.h"

#include <qlocale.h>
#include <qtranslator.h>
#include <qfileinfo.h>
#include <qdir.h>
#include <qtextcodec.h>
#include <qsettings.h>

#include <stdio.h>
#include <stdlib.h>


void myMessageOutput( QtMsgType type, const char *msg ) {
	QString line = QString::fromUtf8(msg);
	switch ( type ) {
		case QtDebugMsg:
#ifndef NO_DEBUG_ON_CONSOLE
			fprintf( stderr, "Debug: %s\n", line.local8Bit().data() );
#endif
			Helper::addLog( line );
			break;
		case QtWarningMsg:
#ifndef NO_DEBUG_ON_CONSOLE
			fprintf( stderr, "Warning: %s\n", line.local8Bit().data() );
#endif
			Helper::addLog( "WARNING: " + line );
			break;
		case QtFatalMsg:
#ifndef NO_DEBUG_ON_CONSOLE
			fprintf( stderr, "Fatal: %s\n", line.local8Bit().data() );
#endif
			Helper::addLog( "FATAL: " + line );
			abort();                    // deliberately core dump
#if QT_VERSION >= 0x040000
		case QtCriticalMsg:
#ifndef NO_DEBUG_ON_CONSOLE
			fprintf( stderr, "Critical: %s\n", line.local8Bit().data() );
#endif
			Helper::addLog( "CRITICAL: " + line );
			break;
#endif
	}
}

void showInfo() {
	qDebug("This is smplayer v. " VERSION " running on "
#ifdef Q_OS_LINUX
           "Linux"
#else
#ifdef Q_OS_WIN
           "Windows"
#else
		   "Other OS"
#endif
#endif
		);
	qDebug("Qt v. " QT_VERSION_STR);
}


int main( int argc, char ** argv ) 
{
	qInstallMsgHandler( myMessageOutput );

	showInfo();

	QApplication a( argc, argv );

	global_init();

	// Get argument with full path
	QString file_to_play="";
	if (a.argc() > 1) {
		//QString f = QString::fromLocal8Bit(a.argv()[1]);
		file_to_play = QTextCodec::codecForLocale()->toUnicode( a.argv()[1] );
	}

	QString app_path = a.applicationDirPath();
	Helper::setAppPath(app_path);
	qDebug( "main: application path: '%s'", app_path.utf8().data());

	// Translator
	QString locale = QLocale::system().name();

	QTranslator translator;
	QString qm_file = Helper::translationPath() + "/smplayer_" + locale +".qm";
	qDebug("main: trying to load translation file: '%s'", qm_file.utf8().data());
	translator.load( qm_file );
	a.installTranslator(&translator);

	qDebug("main: file_to_play: '%s'", file_to_play.utf8().data());

	Gui * w = new Gui(file_to_play);
	//w->show();
	a.connect( &a, SIGNAL( lastWindowClosed() ), &a, SLOT( quit() ) );

	/*
	qDebug("main: file_to_play: '%s'", file_to_play.utf8().data());
	if (!file_to_play.isEmpty()) {
		w->open( file_to_play );
	}
	*/

	int r = a.exec();
	delete w;

	global_end();

	return r;
}
