/*  smplayer, GUI front-end for mplayer.
    Copyright (C) 2007 Ricardo Villalba <rvm@escomposlinux.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "playlist.h"
#include "playlistitem.h"
#include "core.h"

#include <qlistview.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qregexp.h>
#include <qfiledialog.h>
#include <qfileinfo.h>
#include "helper.h"

Playlist::Playlist(Core *c, QWidget * parent, const char* name, WFlags fl)
	: PlaylistBase(parent, name, fl )
{
	core = c;

	listView->addColumn( tr("#") );
	listView->addColumn( tr("Name") );
	listView->addColumn( tr("Length") );
	/*
	listView->setColumnWidthMode(0, QListView::Maximum);
	listView->setColumnWidthMode(1, QListView::Maximum);
	listView->setColumnWidthMode(2, QListView::Maximum);
	listView->setResizeMode(QListView::LastColumn);
	*/
	//listView->setResizePolicy( QListView::AutoOneFit );
	listView->setSorting(0);
	listView->setSortOrder( Qt::Ascending );

	connect( listView, SIGNAL(doubleClicked(QListViewItem*, const QPoint &, int ) ),
             this, SLOT(itemClicked(QListViewItem*)) );

	connect( core, SIGNAL(mediaFinished()), this, SLOT(playNext()) );

	current_item = 0;
}

Playlist::~Playlist() {
}


void Playlist::clear() {
	listView->clear();
	current_item = 0;
}

void Playlist::addItem(int pos, QString filename, QString name, float duration) {
	(void) new PlaylistItem( listView, pos, filename, name, duration );
}

void Playlist::load_m3u(QString file) {
	QRegExp m3u_id("^#EXTM3U");
	QRegExp info("^#EXTINF:(.*),(.*)");

    QFile f( file );
    if ( f.open( IO_ReadOnly ) ) {
		listView->clear();
		int item=1;
		QString filename="";
		QString name="";
		float duration=0;

        QTextStream stream( &f );
        QString line;
        while ( !stream.atEnd() ) {
            line = stream.readLine(); // line of text excluding '\n'
            qDebug( "line: '%s'", line.latin1() );
			if (m3u_id.search(line)!=-1) {
				//#EXTM3U
				// Ignore line
			}
			else
			if (info.search(line)!=-1) {
				duration = info.cap(1).toFloat();
				name = info.cap(2);
				qDebug(" name: '%s', duration: %f", name.latin1(), duration );
			} else {
				filename = line;
				addItem( item, filename, name, duration );
				item++;
				name=""; 
				duration = 0;
			}
        }
        f.close();
		playItem(1);
    }

}

void Playlist::save_m3u(QString file) {
}

void Playlist::load() {
	QString s = QFileDialog::getOpenFileName(
                    "",
                    "Playlists (*.m3u)",
                    this,
                    "open file dialog",
                    tr("Choose a file") );

	if (!s.isEmpty()) {
		load_m3u(s);
	}
}

void Playlist::save() {
	PlaylistItem *item = itemAt(2);
	if (item!=0) qDebug("'%s'", item->filename().latin1());
}

PlaylistItem * Playlist::itemAt( int n ) {
	int i=1;
	QListViewItemIterator it( listView );
	while ( it.current() ) {
		if (n==i) {
			return (PlaylistItem*) it.current();
		}
		++it;
		i++;
	}
	return 0;
}

void Playlist::itemClicked(QListViewItem* item) {
	PlaylistItem *i = (PlaylistItem*) item;

	qDebug("Playlist::itemClicked: pos: %d", i->pos() );

	// Find out the position in the list
	int n=1;
	QListViewItemIterator it( listView );
	while ( it.current() ) {
		PlaylistItem *item2 = (PlaylistItem *) it.current();
		if (item2->pos() == i->pos()) {
			playItem(n);
			break;
		}
		++it;
		n++;
	}
}

void Playlist::playItem( int n ) {
	PlaylistItem *i = itemAt(n);
	if (i!=0) {
		current_item=n;
		listView->setSelected(i, TRUE);

		if (QFileInfo( i->filename() ).exists()) {
			core->openFile( i->filename(), 0 );
		} else {
			core->openStream( i->filename() );
		}
	}
}

void Playlist::playNext() {
	playItem( current_item+1 );
}

void Playlist::playPrev() {
	playItem( current_item-1 );
}
